#!/usr/bin/env python3
# [[file:~/bigtest/bigtest.org::*Download script][Download script:1]]
import os,sys,sqlite3,time,oci,base64,json,shutil
import pandas as pd

if len(sys.argv) != 3 :
    print("{} dbfile outputdirectory".format(__file__))
    sys.exit(1)

conn = sqlite3.connect(sys.argv[1])
dest_dir = sys.argv[2]

config = oci.config.from_file()
os_client = oci.object_storage.ObjectStorageClient(config)
ns = os_client.get_namespace().data
s_endpoint = conn.execute("SELECT value from Globals WHERE name = 'stream-endpoint'").fetchone()[0]
sclient = oci.streaming.StreamClient(config,service_endpoint=s_endpoint)

if os.path.exists(dest_dir):
    print("{} destination directory exists".format(dest_dir))
    sys.exit(1)

os.mkdir(dest_dir)

def create_ocids_csv(destination_path):
    instance_query = '''
    SELECT Region.name,AD.name,Instance.instance_ocid,Instance.creation,Instance.shape,Instance.os
       FROM Instance INNER JOIN AD on AD.id = Instance.ad
       INNER JOIN Region on Region.id = AD.regionid;
    '''
    ocids_df = pd.read_sql_query(instance_query, conn)
    ocids_df.to_csv(os.path.join(destination_path, 'ocids.csv'), header=False, index=False)
    conn.close()

def create_manifest_objects(obj_summaries, dest_dir):
    manifest_file = os.path.join(dest_dir, 'manifest.json')
    data = {'data': oci.util.to_dict(obj_summaries)}
    with open(manifest_file, 'w') as mf:
        json.dump(data, mf)

def list_files_in_bucket(bucket):
    obj_summaries = []
    manifest = os_client.list_objects(ns,bucket,fields="name,size,timeCreated,md5").data
    obj_summaries.extend(manifest.objects)
    while manifest.next_start_with:
        manifest = os_client.list_objects(ns,bucket,start = manifest.next_start_with,
                                          fields="name,size,timeCreated,md5").data
        obj_summaries.extend(manifest.objects)
    return obj_summaries

def download_files_object_store(obj_summaries,bucket, dest_dir):
    for obj_summary in obj_summaries:
        get_obj = os_client.get_object(ns, bucket, obj_summary.name)
        with open(os.path.join(dest_dir, obj_summary.name), 'wb') as f:
            for chunk in get_obj.data.raw.stream(1024 * 1024, decode_content=False):
                f.write(chunk)

def format_log_file(stream,client,logfile):
    m = oci.streaming.models.CreateCursorDetails(type='TRIM_HORIZON',partition="0")
    cursor = sclient.create_cursor(stream,m)

    done = False
    while not done:
        messages = sclient.get_messages(stream,cursor.data.value,limit=100)
        time.sleep(.2) # keeps us from exceeding 5 calls/second
        if not messages.data:
            if cursor.has_next_page:
                cursor = cursor.next_page
            else:
                return

        for message in messages.data:
            logmsg = base64.b64decode(message.value).strip()
            logfile.write(logmsg.decode('utf-8') + '\n')
            logfile.flush()
        cursor.data.value = messages.headers['opc-next-cursor']

if __name__ == "__main__":
    bucketname = conn.execute("SELECT value FROM Globals WHERE name = 'bucket-name';").fetchone()[0]
    stream = conn.execute("SELECT value FROM Globals WHERE name = 'stream';").fetchone()[0]
    create_ocids_csv(dest_dir)
    obj_summaries = list_files_in_bucket(bucketname)
    create_manifest_objects(obj_summaries, dest_dir)
    download_files_object_store(obj_summaries, bucketname, dest_dir)
    format_log_file(stream,sclient,open(dest_dir + '/' + 'logfile.txt','w'))
    if os.access('./config.py',os.F_OK):
        shutil.copyfile('./config.py',dest_dir + '/config.py')  # the runtime set up
# Download script:1 ends here
