#!/usr/bin/env python3
# [[file:~/bigtest/bigtest.org::*Launch the instances][Launch the instances:6]]
ops=[["unix-bench", "unix_bench", "OracleLinux,Ubuntu,CentOS"], ["startup-test", "startup_test", "OracleLinux,Ubuntu,CentOS,Windows"], ["win-test", "win_test", "Windows"], ["win-linpack", "win_linpack", "Windows"], ["win-specint", "win_specint", "Windows"]]
from multiprocessing import Pool
import subroutines
import os,sys,sqlite3,time,oci,base64

from config import *

opnames = list(map(lambda x: x[0],ops))
valid_os = list(map(lambda x: x[2],ops))
if len(sys.argv) != 4:
    s = './%s dbfilename operation (Valid operations: %s) OS (Valid OS: %s'
    sys.exit(s % (sys.argv[0],opnames,valid_os))

if sys.argv[2] not in opnames:
    sys.exit('Invalid operation chosen.  Valid operations: %s' % str(opnames))

for op in ops:
    if sys.argv[2] == op[0] and sys.argv[3] not in op[2]:
        sys.exit('Invalid operation/OS pairing.')

dbname = sys.argv[1]
operation = sys.argv[2]
operatingsystem = sys.argv[3]

public_key_data = open(public_key,'r').read().strip()
conn = sqlite3.connect(dbname)

region_stmt = 'INSERT INTO Region VALUES(null,?,?,?);'
ad_stmt = 'INSERT INTO AD VALUES(null,?,?,?);'

userdataLinux = '''
import os,json,platform,sys
from base64 import b64encode

if sys.version_info[0] < 3:
    from urllib import urlopen
else:
    from urllib.request import urlopen

env = {}

uptime = open('/proc/uptime')
def getuptime():
    uptime.seek(0)
    return uptime.read().split()[0]

def Cmd(cmd):
    def defer():
	if os.system(cmd + " > /dev/null"):
	    print('Command %s failed' % cmd)
	else:
	    print('Command %s succeeded' % cmd)
    return defer

def Yum(packages):
    return Cmd('/usr/bin/sudo /usr/bin/yum install -y ' + packages)

def Chdir(dir):
    def defer():
	os.chdir(dir)

    return defer

def FindResultFile(dir):
    def defer():
	results = None
	for i in os.listdir(dir + '/results'):
	    if i[len(i)-len('.csv'):len(i)] == '.csv':
		env['filename'] = (i,open(dir + '/results/' + i).read())
		return
    return defer

def NullResult():
    def defer():
	env['filename'] = ('null',json.dumps(env['instance']))
    return defer

def UploadResults():
    def defer():
	signer = oci.auth.signers.InstancePrincipalsSecurityTokenSigner()
	osclient = oci.object_storage.ObjectStorageClient(config={},signer=signer)
	osclient.base_client.set_region(env['instance']['metadata']['region'])
	namespace = osclient.get_namespace().data
	id = env['instance']['id']
	shape = env['instance']['shape']
	name = id + ':' + shape + ':' + env['filename'][0]
	data = env['filename'][1]
	osclient.put_object(namespace,env['instance']['metadata']['bucket'],name,data)

    return defer

def TerminateInstance():
    def defer():
	signer = oci.auth.signers.InstancePrincipalsSecurityTokenSigner()
	cc = oci.core.ComputeClient(config={},signer=signer)
	cc.terminate_instance(env['instance']['id'])

    return defer

def Log(message):
    def defer():
	signer = oci.auth.signers.InstancePrincipalsSecurityTokenSigner()
	ep=env['instance']['metadata']['stream-endpoint']
	stream=env['instance']['metadata']['stream']
	sc = oci.streaming.StreamClient(config={},signer=signer,service_endpoint=ep)
	key = b64encode('LaunchLog'.encode()).decode().strip()
	if type(message) == type(lambda : 1):
	    tmp = env['instance']['id'] + ' ' + getuptime() + ' ' + message()
	else:
	    tmp = env['instance']['id'] + ' ' + getuptime() + ' ' + message
	value = b64encode(tmp.encode()).decode().strip()
	tmp = oci.streaming.models.PutMessagesDetailsEntry(key=key,value=value)
	messages = oci.streaming.models.PutMessagesDetails(messages=[tmp])
	try:
	    sc.put_messages(stream,messages)
	except:
	    pass # losing messages with a broken stream is better than an unhandled exception

    return defer

def If(cond,then_,else_ = lambda : None):
    def defer():
	if type(cond) == type(lambda : 1):
	    if cond():
		return then_()
	elif cond:
	    return then_()

	return else_()

    return defer

def Progn(*ops):
    def defer():
	for op in ops:
	    op()
    return defer

env['instance'] = json.load(urlopen('http://169.254.169.254/opc/v1/instance/'))
os_type = platform.platform()
if 'Ubuntu' in os_type:
    Cmd('wget -O /tmp/install.sh  https://raw.githubusercontent.com/oracle/oci-cli/master/scripts/install/install.sh ')()
    Cmd('/usr/bin/sudo bash /tmp/install.sh --install-dir /home/ubuntu/lib/oracle-cli --exec-dir /home/ubuntu --accept-all-defaults')()
    sys.path.insert(0,'/home/ubuntu/lib/oracle-cli/lib/python3.6/site-packages/')
elif 'centos' in os_type:
    Cmd('wget -O /tmp/install.sh  https://raw.githubusercontent.com/oracle/oci-cli/master/scripts/install/install.sh ')()
    Cmd('/usr/bin/sudo bash /tmp/install.sh --install-dir /home/opc/lib/oracle-cli --exec-dir /home/opc --accept-all-defaults')()
    Yum('python-pip')()
    Cmd('sudo pip install oci')()
else:
    Yum('python-pip')()
    Yum('python-oci-cli')()
    Cmd('sudo pip install oci')()

globals()['oci'] = __import__('oci')

Log('Starting test')()

unix_bench = [
    If('Ubuntu' in os_type,
       Progn(Cmd('sudo apt-get update'),
	     Cmd('sudo apt install --yes make gcc'),
	     Chdir('/home/ubuntu')),
       If('oracle' in os_type or 'centos' in os_type,
	  Progn(Yum('git gcc'),Chdir('/home/opc')))),
    Cmd('/usr/bin/git clone https://github.com/kdlucas/byte-unixbench'),
    Chdir('./byte-unixbench/UnixBench'),
    Cmd('/usr/bin/make'),
    Cmd('UB_OUTPUT_CSV=true ./Run'),
    FindResultFile('.'),
    UploadResults(),
    TerminateInstance()
]

startup_test = [
    Log('Creating null result'),
    NullResult(),
    Log('Uploading null result'),
    UploadResults(),
    Log('Terminating instance'),
    TerminateInstance()
]

win_test = [
    Log('Test not appropriate on Linux'),
    TerminateInstance()
]

win_linpack = [
    Log('Test not appropriate on Linux'),
    TerminateInstance()
]

win_specint = [
    Log('Test not appropriate on Linux'),
    TerminateInstance()
]

opMap = {
  "unix-bench" : unix_bench,
  "startup-test" : startup_test,
  "win-test" : win_test,
  "win-linpack" : win_linpack,
  "win-specint" : win_specint,
}


for op in opMap[env['instance']['metadata']['operation']]:
    op()

'''.strip()

userdataWindows = '''
#ps1_sysnative
[Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls12
$env:OCI_CLI_AUTH = 'instance_principal'
Add-Type -assembly "system.io.compression.filesystem"

function Install-OciCli() {
    $wc = new-object System.Net.WebClient
    $url = "https://raw.githubusercontent.com/oracle/oci-cli/master/scripts/install/install.ps1"  
    $wc.DownloadFile($url, "C:\install.ps1")
    powershell -NoProfile -ExecutionPolicy Bypass -file C:\install.ps1 -AcceptAllDefaults
}

$env = New-Object -TypeName PSObject
$instance = Invoke-RestMethod -Uri 'http://169.254.169.254/opc/v1/instance/'

Add-Member -InputObject $env -MemberType NoteProperty -Name 'instance' -Value $instance

$WINTransposeSB = {
    $iterations = 30
    $FilePath = "none"
    Add-Member -InputObject $env -MemberType NoteProperty -Name 'filename' -Value ("null", "$($env.instance | ConvertTo-Json -Compress)")

    #data set up
    $smallDataset = 1..500 | ForEach-Object {
        [pscustomobject] @{Server = "Server$_"; Property = "RAM"; Value = "$_" }
        [pscustomobject] @{Server = "Server$_"; Property = "Cores"; Value = "$_" }
        [pscustomobject] @{Server = "Server$_"; Property = "Disk1"; Value = "$_" }
        #[pscustomobject] @{Server = "Server$_"; Property = "Disk2"; Value = "$_" }
    }
    $largeDataset = 1..750 | ForEach-Object {
        [pscustomobject] @{Server = "Server$_"; Property = "RAM"; Value = "$_" }
        [pscustomobject] @{Server = "Server$_"; Property = "Cores"; Value = "$_" }
        [pscustomobject] @{Server = "Server$_"; Property = "Disk1"; Value = "$_" }
        #[pscustomobject] @{Server = "Server$_"; Property = "Disk2"; Value = "$_" }
    }
    #randomize order in which variations run
    $runs = @()
    1..$iterations | ForEach-Object { $runs += "small"; $runs += "large" }
    $runs = $runs | Sort-Object { Get-Random }

    #run all iterations of every variation
    $runs | ForEach-Object {
        if ($_ -eq "small") {
            $a = $smallDataset
        }
        else {
            $a = $largeDataset
        }
        Write-Host "Starting transpose with input row count=" $a.Count
        # Transpose by "Property", group by "Server" # intersection of "Property X Server = Value" 
        $Duration = Measure-Command {
            $b = @()
            $count = 0;

            foreach ($Property in $a.Property | Select -Unique) {
                $Props = [ordered]@{ Property = $Property.PSObject.Copy() }
                foreach ($Server in $a.Server | Select -Unique) {
                    $Value = ($a.where( { $_.Server -eq $Server -and $_.Property -eq $Property })).Value
                    $Props += @{ $Server = $Value }
                    $count++;
                }
                $b += New-Object -TypeName PSObject -Property $Props
            }
        }

        <# Write-Host "Finished transposing " -ForegroundColor Green -NoNewline
Write-Host "$(($a | Get-Member -MemberType Properties).count)/$($a.Count)" -ForegroundColor Yellow -NoNewline Write-Host " columns/rows into " -ForegroundColor Green -NoNewline
Write-Host "$(($b | Get-Member -MemberType Properties).count)/$($b.Count)" -ForegroundColor Yellow -NoNewline Write-Host " columns/rows in " -ForegroundColor Green -NoNewline
Write-Host "Total Time:" $Duration -ForegroundColor Yellow -NoNewline #>

        $instance_id_parts = $env.instance.id.Split(".")
        $adPart = $instance_id_parts[$instance_id_parts.Length - 1].ToCharArray()
        $availability_domain = "AD1"
        Switch ($adPart[7]) {
            "r" { $availability_domain = "AD1" }
            "s" { $availability_domain = "AD2" }
            "t" { $availability_domain = "AD3" }
        }
        #$csvMeasures = $a | Measure-Object
        $myprop = @{
            seconds       = $Duration.TotalSeconds
            instance_id   = $env.instance.id
            file          = "$FilePath"
            region        = $env.instance.canonicalRegionName
            shape         = $env.instance.shape
            compartmentId = $env.instance.compartmentId
            AD            = $availability_domain
            rows          = $a.Count
        }
        $myobj = New-Object psobject -Property $myprop

        #write line to csv for upload later
        Export-Csv -InputObject $myobj -Path "C:\\temp.txt" -Append -NoTypeInformation
    }
}

$LinpackTestSB = {
    $benchmark = @"
Shared-memory version of Intel(R) Distribution for LINPACK* Benchmark. *Other names and brands may be claimed as the property of others.
Sample data file lininput_xeon64.
14                     # number of tests
1000 2000 5000 10000 15000 18000 20000 22000 25000 26000 27000 30000 35000 40000 # problem sizes
1000 2000 5008 10000 15000 18008 20016 22008 25000 26000 27000 30000 35000 40000 # leading dimensions
4 2 2 2 2 2 2 2 2 2 1 1 1 1 1 # times to run a test
4 4 4 4 4 4 4 4 4 4 4 1 1 1 1 # alignment values (in KBytes)
"@
    Write-Host "Downloading Linpack"
    $progressPreference = 'silentlyContinue'
    Invoke-WebRequest "https://software.intel.com/sites/default/files/managed/52/9d/w_mklb_p_2019.6.005.zip" -OutFile "c:\\linpack.zip"
    $progressPreference = 'Continue'
    Set-Location -Path "C:\\"
    Write-Host "Extracting Linpack"
    Expand-Archive "linpack.zip"
    Write-Host "Moving to Linpack benchmark directory"
    Set-Location -Path "C:\\linpack\\w_mklb_p_2019.6.005\\benchmarks_2019\\windows\\mkl\\benchmarks\\linpack"
    Write-Host "Replacing benchmark file"
    Set-Content -Path "lininput_xeon64" -Value $benchmark
    Write-Host "Running Linpack benchmark"
    .\\runme_xeon64.bat
    Write-Host "Copying results to upload location"
    Copy-Item -Path "win_xeon64.txt" -Destination "C:\\temp.txt"
}

$SetupSpecIntSB = {
    $ns = (C:\\Users\\cloudbase-init\\bin\\oci.exe os ns get | out-string | ConvertFrom-Json).data
    # Download and extract spec int from object store
    $spec_zip_file = "C:\\spec_int_win.zip"
    $spec_int_root = "C:\\"

    $args1 = @("--region", $($env.instance.metadata.region), "os", "object", "get", "-bn", $($env.instance.metadata.specintbucket), "--name", $($env.instance.metadata.specintfilename), "--file", $spec_zip_file)
    Start-Process -Wait -NoNewWindow -RedirectStandardError C:\error.txt -FilePath 'C:\\Users\\cloudbase-init\\bin\\oci.exe' -Argumentlist $args1
    [IO.Compression.ZipFile]::ExtractToDirectory($spec_zip_file, $spec_int_root)

    # Create and attach a new volume
    $args_bv_create = @("--region", $($env.instance.canonicalRegionName), "bv", "volume", "create", "--availability-domain", $($env.instance.availabilityDomain), "-c", $($env.instance.compartmentId), "--display-name", "spec_int_$((Get-Date).ToFileTimeUtc())", "--size-in-gbs", "200")
    Start-Process -Wait -NoNewWindow -RedirectStandardError C:\\vol_create_error.txt -RedirectStandardOutput C:\\vol_create.txt -FilePath 'C:\\Users\\cloudbase-init\\bin\\oci.exe' -Argumentlist $args_bv_create
    $vol_create = Get-Content C:\\vol_create.txt | ConvertFrom-Json
    sleep -Seconds 30

    #Attach volume
    $args_bv_attach = @("--region", $($env.instance.canonicalRegionName), "compute", "volume-attachment", "attach-paravirtualized-volume", "--instance-id",$($env.instance.id), "--volume-id", $($vol_create.data.id))
    Start-Process -Wait -NoNewWindow -RedirectStandardError C:\\vol_attach_error.txt -RedirectStandardOutput C:\\vol_attach.txt -FilePath 'C:\\Users\\cloudbase-init\\bin\\oci.exe' -Argumentlist $args_bv_attach
    sleep -Seconds 30
}

$SetupScheduledTaskSB={
    $Trigger= New-ScheduledTaskTrigger -AtStartup
    $User= "NT AUTHORITY\SYSTEM"
    $Action= New-ScheduledTaskAction -Execute "PowerShell.exe" -Argument "C:\\run_spec.ps1"
    Register-ScheduledTask -TaskName "StartupScript_PS" -Trigger $Trigger -User $User -Action $Action -RunLevel Highest -Force
}

$StartSpecIntRunSB = {
    $args_install = @("-File", "C:\initial_setup.ps1")
    Start-Process -Wait -NoNewWindow -RedirectStandardError C:\\initial_setup_error.txt -RedirectStandardOutput C:\\nitial_setup.txt -FilePath "PowerShell.exe" -Argumentlist $args_install
}

$NullResultSB = {
    Add-Member -InputObject $env -MemberType NoteProperty -Name 'filename' -Value ("null", "$($env.instance | ConvertTo-Json -Compress)")
    $data = $env.filename[1]
    $tempresult = $(Join-Path -Path 'C:' -ChildPath "temp.txt")
    $data | Out-File -FilePath $tempresult -Encoding UTF8
}

$UploadResultSB = {
    $ns = (C:\\Users\\cloudbase-init\\bin\\oci.exe os ns get | out-string | ConvertFrom-Json).data
    $id = $env.instance.id
    $shape = $env.instance.shape
    $name = $id + ':' + "$shape" + ':' + $($env.filename[0])
    $tempresult = $(Join-Path -Path 'C:' -ChildPath "temp.txt")
    $args1 = @("--region", $($env.instance.metadata.region), "os", "object", "put", "-bn", $($env.instance.metadata.bucket), "--name", $name, "--file", $tempresult)
    Start-Process -Wait -NoNewWindow -RedirectStandardError C:\error.txt -FilePath 'C:\\Users\\cloudbase-init\\bin\\oci.exe' -Argumentlist $args1
}

$TerminateInstanceSB = {
    $id = $id = $env.instance.id
    $args2 = @("compute", "instance", "terminate", "--force", "--instance-id", $id)
    Start-Process -Wait -NoNewWindow -RedirectStandardError C:\error_terminate.txt -FilePath 'C:\\Users\\cloudbase-init\\bin\\oci.exe' -Argumentlist $args2
}

$startup_test = ($NullResultSB, $UploadResultSB, $TerminateInstanceSB)
$win_test = ($WinTransposeSB, $UploadResultSB, $TerminateInstanceSB)
$win_linpack = ($LinpackTestSB, $UploadResultSB, $TerminateInstanceSB)
$win_specint = ($SetupSpecIntSB, $SetupScheduledTaskSB, $StartSpecIntRunSB)

$opMap = @{
    "startup-test" = $startup_test;
    "win-test"     = $win_test;
    "win-linpack"  = $win_linpack;
    "win-specint"  = $win_specint
}
Install-OciCli
$opMap[$env.instance.metadata.operation] | ForEach-Object { Invoke-Command -ScriptBlock $_ }
'''.strip()

if operatingsystem == 'Ubuntu':
    userdatapythonenv = '#!/usr/bin/env python3\n' + userdataLinux
    userdata_enc = base64.b64encode(userdatapythonenv.encode()).decode("ascii")
elif operatingsystem in ['OracleLinux','CentOS']:
    userdatapythonenv = '#!/usr/bin/env python\n' + userdataLinux
    userdata_enc = base64.b64encode(userdatapythonenv.encode()).decode("ascii")
else:
    userdata_enc = base64.b64encode(userdataWindows.encode()).decode("ascii")

config = oci.config.from_file()

subnets = {}

for (region,ads) in regions:
    config['region'] = region
    nc = oci.core.VirtualNetworkClient(config)
    vcn_details = oci.core.models.CreateVcnDetails(compartment_id=compartment,
                                                   cidr_block='192.168.0.0/16')

    vcn = nc.create_vcn(vcn_details)
    ig_details = oci.core.models.CreateInternetGatewayDetails(compartment_id=compartment,
                                                              vcn_id=vcn.data.id,
                                                              is_enabled=True)
    ig = nc.create_internet_gateway(ig_details)
    conn.execute(region_stmt,(vcn.data.id,ig.data.id,region))
    conn.commit()

    defroutetable = vcn.data.default_route_table_id
    rr=oci.core.models.RouteRule(destination='0.0.0.0/0',destination_type='CIDR_BLOCK',
                                 network_entity_id=ig.data.id)
    nc.update_route_table(defroutetable,
                          oci.core.models.UpdateRouteTableDetails(route_rules=[rr]))
    rowid=conn.execute('SELECT last_insert_rowid() from Region;').fetchone()[0]

    count = 1
    for ad in ads.split(','):
        cidrblock='192.168.%d.0/24' % count
        count = count + 1
        details = oci.core.models.CreateSubnetDetails(compartment_id=compartment,
                                                      availability_domain=ad,
                                                      vcn_id=vcn.data.id,
                                                      cidr_block=cidrblock)
        subnet = nc.create_subnet(details)
        subnets[(region,ad)] = subnet.data.id
        conn.execute(ad_stmt,(rowid,subnet.data.id,ad))
        conn.commit()

conn.execute("INSERT INTO Globals VALUES('starttime',strftime('%s','now'));")
conn.commit()

def image_lookup(region,operatingsystem):
    dispatch = {'OracleLinux' : images,
                'Ubuntu' : ubuntuimages,
                'CentOS' : centosimages,
                'Windows' : windowsimages}
    for (region_,image) in dispatch[operatingsystem]:
        if region == region_:
            return image

    return None

def check_for_termination(client,instances,sleeptime=30):
    tmp = []
    for i in instances:
        try:
            if client.get_instance(i.data.id).data.lifecycle_state != 'TERMINATED':
                tmp.append(i)
        except:     # act like it wasn't terminated...we'll get it next time if it is
            tmp.append(i)
    time.sleep(sleeptime)   # don't hammer the get_instance API
    return tmp
            
def launchinstance(instanceiterator):
    cc = None
    dbconn = sqlite3.connect(dbname)
    metadata={
        'ssh_authorized_keys' : public_key_data,
        'user_data' : userdata_enc,
        'operation' : operation,
        'operatingsystem' : operatingsystem,
        'region' : homeregion,
        'bucket' : dbconn.execute("SELECT value FROM Globals WHERE name = 'bucket-name';").fetchone()[0],
        'stream' : dbconn.execute("SELECT value FROM Globals WHERE name = 'stream';").fetchone()[0],
        'stream-endpoint' : dbconn.execute("SELECT value FROM Globals WHERE name = 'stream-endpoint';").fetchone()[0],
        'specintbucket' : specintbucket,
        'specintfilename' : specintfilename
    }
    idobjs = []
    ads = {}
    adrow = None
    for (region,ad,count,name) in instanceiterator:
        if adrow == None:
            adrow = dbconn.execute("SELECT id from AD WHERE name = ?",(ad,)).fetchone()[0]
            
        if cc == None:
            config['region'] = region # safe as we're multiprocessing
            cc = oci.core.ComputeClient(config)

            image = image_lookup(region,operatingsystem)
            if image == None:
                sys.stderr.write("Failed looking up image for region: %s.\n" % region)
                return None
            
        instancedetails = oci.core.models.LaunchInstanceDetails(availability_domain=ad,
                                                                compartment_id=compartment,
                                                                subnet_id=subnets[(region,ad)],
                                                                shape=shape,
                                                                image_id=image,
                                                                display_name=name,
                                                                metadata=metadata)
        idobjs.append(instancedetails)
    runninginstances = []
    launched = 0

    instance_stmt = 'INSERT INTO Instance(id,ad,instance_ocid,name,shape,os) VALUES(null,?,?,?,?,?)'
    fails = 0
    while launched != len(idobjs):
        if len(runninginstances) != limit:
            try:
                instance = cc.launch_instance(idobjs[launched])
                displayname=idobjs[launched].display_name
                launched = launched + 1
                runninginstances.append(instance)
                dbconn.execute(instance_stmt,(adrow,instance.data.id,displayname,shape,operatingsystem))
                dbconn.commit()
                fails = 0
                print('Launched: ' + instance.data.id)
            except:
                sys.stderr.write('Launch exception(%s): trying again\n' % (repr(sys.exc_info())))
                fails = fails + 1
                if(fails == 10):
                    cleanup(runninginstances,region,cc)
                    return True       # probably not going to clear up if we've been trying for ~2 minutes
                time.sleep(2 * fails) # don't hammer the API too hard
        else:
            print('Checking for terminated instances.')
            runninginstances = check_for_termination(cc,runninginstances)

    cleanup(runninginstances,region,cc)
    return True

def cleanup(runninginstances,region,cc):
    while len(runninginstances):
        print('Waiting for instances to terminate in %s.  Remaining: %d' % (region,len(runninginstances)))
        runninginstances = check_for_termination(cc,runninginstances,15)

# run serially if we're debugging an issue
if debug == True:
    for instanceiterator in subroutines.generate_instances(launches,regions):
        launchinstance(instanceiterator)
else:
    res = subroutines.shard_instances(subroutines.generate_instances(launches,regions),launchinstance)

conn.execute("INSERT INTO Globals VALUES('endtime',strftime('%s','now'));")
conn.commit()
conn.close()
if debug == False:
    print(res)
# Launch the instances:6 ends here
