#!/usr/bin/env python3
# [[file:~/bigtest/bigtest.org::*Test viewer][Test viewer:1]]
from tkinter import *
from tkinter import ttk
from multiprocessing import Pool
import sqlite3,sys,os,oci

config = oci.config.from_file()

nwclients = {}
osclients = {}
computeclients = {}

if len(sys.argv) != 2:
    sys.exit('%s dbfilename' % sys.argv[0])

if os.access(sys.argv[1],os.F_OK) == False:
    sys.exit("Dbfilename(%s) does not exist" % sys.argv[1])

refresh_globals = lambda : dbconn.execute('SELECT * from Globals;').fetchall()
refresh_regions = lambda : dbconn.execute('SELECT * FROM Region;').fetchall()
refresh_ADs = lambda : dbconn.execute('SELECT * FROM AD;').fetchall()
refresh_instances = lambda : dbconn.execute('SELECT * FROM Instance;').fetchall()
subnets = {}
regionvars = {}

def entry(tab,width,col,row,text):
    e = Entry(tab,font=('Arial',12),width=width)
    e.insert(END,text)
    e.config(state='readonly')
    e.grid(column=col,row=row,sticky=E)

def build_global_tab(tab):
    Label(tab,text='Global information:',font=('Arial Bold',16)).grid(column=0,row=0)
    Label(tab,text='Name',font=('Arial Bold',12)).grid(column=0,row=1)
    Label(tab,text='Value',font=('Arial Bold',12)).grid(column=1,row=1)
    row = 2
    nwid = 0
    vwid = 0

    for i in globals:
        if len(i[0]) > nwid: nwid = len(i[0])
        if len(i[1]) > vwid: vwid = len(i[1])

    for i in globals:
        entry(tab,nwid,0,row,i[0])
        entry(tab,vwid,1,row,i[1])
        row = row + 1

def delete_region(tab):
    global regions

    for region in regionvars.keys():
        if regionvars[region].get() == True:
           client = nwclients[region[3]]
           defroutetable = client.get_vcn(region[1]).data.default_route_table_id
           model = oci.core.models.UpdateRouteTableDetails(route_rules=[])
           try:
               client.update_route_table(defroutetable,model)
               client.delete_internet_gateway(region[2])
               client.delete_vcn(region[1])
           except Exception as e:
               print(e) 
           dbconn.execute('DELETE FROM Region WHERE id = ?;',(region[0],))
           dbconn.commit()
 
    regions = refresh_regions()
    build_region_tab(tab)

def build_region_tab(tab):
    global regionvars
    for widget in tab.winfo_children():
         widget.destroy()

    Label(tab, text='Region information:',font=('Arial Bold',16)).grid(column=0,row=0)
    Label(tab,text='Name',font=('Arial Bold',12)).grid(column=0,row=1)
    Label(tab,text='VCN',font=('Arial Bold',12)).grid(column=1,row=1)
    Label(tab,text='Internet Gateway',font=('Arial Bold',12)).grid(column=2,row=1)
    row = 2

    nwid = 0
    vcnwid = 0
    igwid = 0

    for i in regions:
       if len(i[3]) > nwid: nwid = len(i[3])
       if len(i[1]) > vcnwid: vcnwid = len(i[1])
       if len(i[2]) > igwid: igwid = len(i[2])

    for i in regions:
        tmp = BooleanVar()
        tmp.set(False)
        regionvars[i] = tmp
        Checkbutton(tab,text='',var=tmp).grid(column=0,row=row,sticky=E)
        entry(tab,nwid,1,row,i[3])
        entry(tab,vcnwid,2,row,i[1])
        entry(tab,igwid,3,row,i[2])
        row=row + 1

    txt = 'Delete marked region data'
    Button(tab, text=txt, command=lambda : delete_region(tab)).grid(column=0,row=row)

def delete_ad(tab):
    global ADs
    for subnet in subnets.keys():
        if subnets[subnet].get() == True:
             print('Deleting: %s' % subnet[2])
             for i in regions:
                 if i[0] == subnet[1]:
                     try:
                         nwclients[i[3]].delete_subnet(subnet[2]) 
                     except Exception as e:
                         print(e) 
                     dbconn.execute('DELETE FROM AD WHERE subnet_ocid = ?',(subnet[2],))
                     dbconn.commit()
                     break
   
    ADs = refresh_ADs()
    build_ad_tab(tab)

def build_ad_tab(tab):
    global subnets

    for widget in tab.winfo_children():
         widget.destroy()

    subnets = {}
    Label(tab, text='AD information:',font=('Arial Bold',16)).grid(column=0,row=0)
    Label(tab,text='Name',font=('Arial Bold',12)).grid(column=1,row=1)
    Label(tab,text='Subnet',font=('Arial Bold',12)).grid(column=2,row=1)
    row = 2
  
    subnetwid = 0
    nwid = 0

    for i in ADs:
        if len(i[3]) > nwid: nwid = len(i[3])
        if len(i[2]) > subnetwid: subnetwid = len(i[2])

    for i in ADs:
        tmp = BooleanVar()
        tmp.set(False)
        subnets[i] = tmp
        Checkbutton(tab,text='',var=tmp).grid(column=0,row=row,sticky=E)
        entry(tab,nwid,1,row,i[3])
        entry(tab,subnetwid,2,row,i[2])
        row=row + 1
    Button(tab, text="Delete marked ADs", command=lambda : delete_ad(tab)).grid(column=0,row=row)

def update_instance_tab(tab):
    global instances
    instances = refresh_instances()
    build_instance_tab(tab)

def update_bucket_tab(tab):
    build_bucket_tab(tab)

def build_bucket_tab(tab):
    Label(tab, text='Bucket information:',font=('Arial Bold',16)).grid(column=0,row=0)
    Button(tab,text='Refresh object list',command=lambda : update_bucket_tab(tab)).grid(column=1,row=0)
    client = osclients[homeregion]
    ns = client.get_namespace().data
    # the limit should be handled more elegantly
    res = client.list_objects(ns,bucketname,limit=1000).data.objects

    nwid = 0
    for file in res:
        if len(file.name) > nwid: nwid = len(file.name)

    row = 2
    entry(tab,nwid,0,row-1,'Name')
    for file in res:
        entry(tab,nwid,0,row,file.name)
        row = row + 1

def build_instance_tab(tab):
    Label(tab, text= 'Instance information:',font=('Arial Bold',16)).grid(column=0,row=0)
    txt = 'Refresh instance info'
    update = lambda : update_instance_tab(tab)
    Button(tab, text=txt, command=update).grid(column=1,row=0)

    entry(tab,len("Instance OCID"),0,1,"Instance OCID")
    entry(tab,len("Creation timestamp"),1,1,"Creation timestamp")
    entry(tab,len("Region"),2,1,"Region")
    entry(tab,len("Provisioning"),3,1,"Status")
    row=2
     
    iwid = 0
    rwid = len('eu-frankfurt-1') + 3 # gross hack
    swid = len('PROVISIONING') + 3   # YAGH
    for instance in instances:
        cwid = len(instance[4])   
        if len(instance[2]) > iwid: iwid = len(instance[2])

    instance_state = collect_instance_state()
    count = 0
    for instance in instances:
       entry(tab,iwid,0,row,instance[2])
       entry(tab,cwid,1,row,instance[4])
       region = instance[3].split('-')[1:4]
       region = region[0] + '-' + region[1] + '-' + region[2]
       
       entry(tab,rwid,2,row,region)
       entry(tab,swid,3,row,instance_state[count])
       row = row + 1
       count = count +1

def collect_instance_state():
    regioninfo = {}
    results = []
    lookup = lambda region,ocid,page: computeclients[region].list_instances(ocid,page=page)

    for instance in instances:
        region = instance[3].split('-')[1:4]
        region = region[0] + '-' + region[1] + '-' + region[2]
        if not(region in regioninfo):
            for (key,value) in globals:
                if key == 'compartment':
                    done = False
                    page=None
                    regioninfo[region] = []
                    while done != True:
                        data = lookup(region,value,page)
                        if data.has_next_page == False:
                            done = True
                        else:
                            page = data.next_page

                        for instance_ in data.data:
                            regioninfo[region].append(instance_)

        for elt in regioninfo[region]:
             if elt.id == instance[2]:
                 results.append(elt.lifecycle_state)
    return results
    
try:
   dbconn = sqlite3.connect(sys.argv[1])
   globals = refresh_globals()
   regions = refresh_regions()
   ADs = refresh_ADs()
   homeregion = dbconn.execute("SELECT value FROM Globals WHERE name = 'homeregion';").fetchone()[0]
   bucketname = dbconn.execute("SELECT value FROM Globals WHERE name = 'bucket-name';").fetchone()[0]
   instances = refresh_instances()
   for (id,VCN,IG,name) in regions:
        config['region'] = name
        nwclients[name] = oci.core.VirtualNetworkClient(config)
        osclients[name] = oci.object_storage.ObjectStorageClient(config)
        computeclients[name] = oci.core.ComputeClient(config)
except:
   sys.exit('Dbfilename(%s) was not a database or incorrect schema' % sys.argv[1])

window = Tk()
window.title("bigtest's cleanup application")
window.geometry('1400x600')
tab_control = ttk.Notebook(window)
globaltab = ttk.Frame(tab_control)
regiontab = ttk.Frame(tab_control)
adtab = ttk.Frame(tab_control)
instancetab = ttk.Frame(tab_control)
buckettab = ttk.Frame(tab_control)

instancetab.grid()
instancetab.rowconfigure(0,weight=1)
instancetab.columnconfigure(0,weight=1)
instancecanvas = Canvas(instancetab)
instancecanvas.grid(row=0,column=0,sticky="nsew")
instancelist = ttk.Frame(instancecanvas)
instancecanvas.create_window(0,0,window=instancelist,anchor='nw')
s = Scrollbar(instancetab,orient=VERTICAL)
s.config(command=instancecanvas.yview)
instancecanvas.config(yscrollcommand=s.set)
s.grid(row=0,column=1,sticky="ns")

buckettab.grid()
buckettab.rowconfigure(0,weight=1)
buckettab.columnconfigure(0,weight=1)
bucketcanvas = Canvas(buckettab)
bucketcanvas.grid(row=0,column=0,sticky="nsew")
bucketlist = ttk.Frame(bucketcanvas)
bucketcanvas.create_window(0,0,window=bucketlist,anchor='nw')
ss = Scrollbar(buckettab,orient=VERTICAL)
ss.config(command=bucketcanvas.yview)
bucketcanvas.config(yscrollcommand=ss.set)
ss.grid(row=0,column=1,sticky="ns")


def scrollinstances(event):
    instancecanvas.configure(scrollregion=instancecanvas.bbox("all"))

def scrollbucket(event):
    bucketcanvas.configure(scrollregion=bucketcanvas.bbox("all"))

instancecanvas.bind("<Configure>",scrollinstances)
bucketcanvas.bind("<Configure>",scrollbucket)

tab_control.add(globaltab, text='Globals')
tab_control.add(regiontab, text='Regions')
tab_control.add(adtab, text='ADs')
tab_control.add(instancetab, text='Instances')
tab_control.add(buckettab,text='Bucket Info')

build_global_tab(globaltab)
build_region_tab(regiontab)
build_ad_tab(adtab)
build_instance_tab(instancelist)
build_bucket_tab(bucketlist)

tab_control.pack(expand=1, fill='both')
window.mainloop()
# Test viewer:1 ends here
