#!/usr/bin/env python3
# [[file:~/bigtest/bigtest.org::*Creation script][Creation script:1]]
import os,sys,sqlite3,time,oci

from config import tenancy,homeregion,compartment,dgname,cname

# uniquely name the run database
sfx = str(int(time.time()))
dbname = 'run.' + sfx
conn = sqlite3.connect(dbname)

schema = '''
CREATE TABLE Globals (
    name TEXT NOT NULL,
    value TEXT NOT NULL
);
CREATE TABLE Region (
 id INTEGER PRIMARY KEY,
 VCN_ocid TEXT NOT NULL,
 IG_ocid TEXT NOT NULL,
 name TEXT NOT NULL
);
CREATE TABLE AD (
 id INTEGER PRIMARY KEY,
 regionid INTEGER,
 subnet_ocid TEXT NOT NULL,
 name TEXT NOT NULL
);
CREATE TABLE Instance (
 id INTEGER PRIMARY KEY,
 ad INTEGER,
 instance_ocid TEXT NOT NULL,
 name TEXT NOT NULL,
 shape TEXT NOT NULL,
 os TEXT NOT NULL,
 creation DATETIME DEFAULT CURRENT_TIMESTAMP
);
'''

conn.executescript(schema)
conn.commit()

config = oci.config.from_file()
config['region'] = homeregion

cc= oci.identity.IdentityClient(config)
match = "ALL {instance.compartment.id = '%s'}" % compartment
description='Dynamic-group for test'
cdg_details = oci.identity.models.CreateDynamicGroupDetails(compartment_id=tenancy,
                                                            name=dgname + '-' + sfx,
                                                            matching_rule=match,
                                                            description=description)
dgid = cc.create_dynamic_group(cdg_details).data.id
conn.execute("INSERT INTO Globals VALUES('dynamic-group-id',?);",(dgid,))
conn.commit()

active = False

# this only takes a few seconds
while active != True:
    if cc.get_dynamic_group(dgid).data.lifecycle_state == 'ACTIVE':
        active = True
    else:
        time.sleep(2)

active = False

statements = [
    'ALLOW dynamic-group %s to manage instance-family in compartment %s' % (dgname + '-' + sfx,cname),
    'ALLOW dynamic-group %s to manage volume-family in compartment %s' % (dgname + '-' + sfx,cname),
    'ALLOW dynamic-group %s to read buckets in compartment %s' % (dgname + '-' + sfx,cname),
    'ALLOW dynamic-group %s to manage objects in compartment %s' % (dgname + '-' + sfx,cname),
    'ALLOW dynamic-group %s to use stream-push in compartment %s' % (dgname + '-' + sfx,cname),
]

policy_details = oci.identity.models.CreatePolicyDetails(compartment_id=tenancy,
                                                         name='testpolicy-' + sfx,
                                                         statements=statements,
                                                         description='test policy')
pid = cc.create_policy(policy_details).data.id
conn.execute("INSERT INTO Globals VALUES('dynamic-group-policy',?);",(pid,))
conn.commit()

while active != True:
    if cc.get_policy(pid).data.lifecycle_state == 'ACTIVE':
        active = True
    else:
        time.sleep(2)

osclient = oci.object_storage.ObjectStorageClient(config)
namespace = osclient.get_namespace().data
cb_details = oci.object_storage.models.CreateBucketDetails(name='testbucket-' + sfx,
                                                           compartment_id=compartment)
bucket = osclient.create_bucket(namespace,cb_details)

s_admin_client = oci.streaming.StreamAdminClient(config)
ac_details = oci.streaming.models.CreateStreamDetails(name='teststream-' + sfx,
                                                      partitions=1,
                                                      compartment_id=compartment)

stream = s_admin_client.create_stream(ac_details)

conn.execute("INSERT INTO Globals VALUES('bucket-name',?);",('testbucket-' + sfx,))
conn.execute("INSERT INTO Globals VALUES('compartment',?);",(compartment,))
conn.execute("INSERT INTO Globals VALUES('homeregion',?);",(homeregion,))
conn.execute("INSERT INTO Globals VALUES('stream',?);",(stream.data.id,))
conn.execute("INSERT INTO Globals VALUES('stream-endpoint',?);",(stream.data.messages_endpoint,))
conn.commit()

print('Database file: ' + dbname)
# Creation script:1 ends here
