#!/usr/bin/env python3
# [[file:~/bigtest/bigtest.org::*Teardown script][Teardown script:1]]
import os,sys,sqlite3,time,oci
from multiprocessing import Pool

config = oci.config.from_file()
conn = sqlite3.connect(sys.argv[1])

instance_query = '''
SELECT Region.name,Instance.instance_ocid from Region,AD,Instance WHERE
            Instance.ad = AD.id and Region.id = AD.regionid;
'''
instances = conn.execute(instance_query).fetchall()

def shutdown(instance):
    config['region'] = instance[0]
    start = time.time()
    cc = oci.core.ComputeClientCompositeOperations(oci.core.ComputeClient(config))
    try:
        cc.terminate_instance_and_wait_for_state(instance[1],['TERMINATED'])
    except:
        pass    # handle the case where the Instance termination fails for some reason
    return (time.time() - start)

if len(instances):
    res = Pool(processes=30).map(shutdown,instances)
    print('Instances terminated: ' + repr(res))
    conn.execute('DELETE FROM Instance;')
    conn.commit()

for row in conn.execute('SELECT * FROM Region;'):
     config['region'] = row[3]
     ad_stmt = 'SELECT id,subnet_ocid FROM AD WHERE regionid = ?'
     nc = oci.core.VirtualNetworkClient(config)
     for ad in conn.execute(ad_stmt,(row[0],)):
          nc.delete_subnet(ad[1])

     conn.execute('DELETE FROM AD WHERE regionid = ?;',(row[0],))
     conn.execute('DELETE FROM Region WHERE id = ?;',(row[0],))
     conn.commit()
     defroutetable = nc.get_vcn(row[1]).data.default_route_table_id
     nc.update_route_table(defroutetable,oci.core.models.UpdateRouteTableDetails(route_rules=[]))
     nc.delete_internet_gateway(row[2])
     nc.delete_vcn(row[1])
# Teardown script:1 ends here
